/*
  * The MIT License
  *
  * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
  * Further enhancement before move to Codehaus sponsored and donated by Lakeside A/S (http://www.lakeside.dk)
  *
  * Copyright (c) to all contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
  * the Software without restriction, including without limitation the rights to
  * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
  * of the Software, and to permit persons to whom the Software is furnished to do
  * so, subject to the following conditions:
  *
  * The above copyright notice and this permission notice shall be included in all
  * copies or substantial portions of the Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  *
  * $HeadURL: https://svn.codehaus.org/mojo/trunk/mojo/chronos/chronos-report-maven-plugin/src/main/java/org/codehaus/mojo/chronos/common/model/ResponsetimeSamples.java $
  * $Id: ResponsetimeSamples.java 15769 2012-01-24 12:09:52Z soelvpil $
  */
package org.codehaus.mojo.chronos.common.model;

import java.util.Iterator;

public class SortedList<V extends Comparable<V>>
    implements Iterable<V>
{
    private static final boolean RED = false;

    private static final boolean BLACK = true;

    private Node<V> root;

    public void add( V value )
    {
        Node<V> t = root;
        if ( t == null )
        {
            root = new Node<V>( value, null );
            t = root;
        }
        while ( true )
        {
            int cmp = value.compareTo( t.value );
            if ( cmp == 0 )
            {
                t.value = value;
                return;
            }
            else if ( cmp < 0 )
            {
                if ( t.left != null )
                {
                    t = t.left;
                }
                else
                {
                    t.left = new Node<V>( value, t );
                    fixAfterInsertion( t.left );
                    return;
                }
            }
            else
            {
                if ( t.right != null )
                {
                    t = t.right;
                }
                else
                {
                    t.right = new Node<V>( value, t );
                    fixAfterInsertion( t.right );
                    return;
                }
            }
        }
    }

    public V remove( V value )
    {
        Node<V> p = getNode( value );
        if ( p == null )
        {
            return null;
        }
        V oldValue = p.value;
        deleteNode( p );
        return oldValue;
    }

    private void deleteNode( Node<V> p )
    {
        if ( p.left != null && p.right != null )
        {
            Node<V> s = getSuccessor( p );
            p.value = s.value;
            p = s;
        }
        Node<V> replacement = ( p.left != null ) ? p.left : p.right;
        if ( replacement != null )
        {
            // link replacement to parent
            replacement.parent = p.parent;
            if ( p.parent == null )
            {
                root = replacement;
            }
            else if ( p == p.parent.left )
            {
                p.parent.left = replacement;
            }
            else
            {
                p.parent.right = replacement;
            }
            // null out links so they may be used by fixAfterDeletion
            p.left = p.right = p.parent = null;
            fixAfterDeletion( replacement );
        }
        else if ( p.parent == null )
        {
            root = null;
        }
        else
        {
            // no children. Use self as phantom replacement and unlink
            if ( p.color == BLACK )
            {
                fixAfterDeletion( p );
            }

            if ( p.parent != null )
            {
                if ( p == p.parent.left )
                {
                    p.parent.left = null;
                }
                else if ( p == p.parent.right )
                {
                    p.parent.right = null;
                }
                p.parent = null;
            }

        }

    }

    private void fixAfterDeletion( Node<V> x )
    {
        while ( x != root && colorOf( x ) == BLACK )
        {
            if ( x == leftOf( parentOf( x ) ) )
            {
                Node<V> sibling = rightOf( parentOf( x ) );
                if ( colorOf( sibling ) == RED )
                {
                    setColor( sibling, BLACK );
                    setColor( parentOf( x ), RED );
                    rotateLeft( parentOf( x ) );
                    sibling = rightOf( parentOf( x ) );
                }
                if ( colorOf( leftOf( sibling ) ) == BLACK && colorOf( rightOf( sibling ) ) == BLACK )
                {
                    setColor( sibling, RED );
                    x = parentOf( x );
                }
                else
                {
                    if ( colorOf( rightOf( sibling ) ) == BLACK )
                    {
                        setColor( leftOf( sibling ), BLACK );
                        setColor( sibling, RED );
                        rotateRight( sibling );
                        sibling = rightOf( parentOf( x ) );
                    }
                    setColor( sibling, colorOf( parentOf( x ) ) );
                    setColor( parentOf( x ), BLACK );
                    setColor( rightOf( sibling ), BLACK );
                    rotateLeft( parentOf( x ) );
                    x = root;
                }
            }
            else
            {
                Node<V> sibling = leftOf( parentOf( x ) );
                if ( colorOf( sibling ) == RED )
                {
                    setColor( sibling, BLACK );
                    setColor( parentOf( x ), RED );
                    rotateRight( parentOf( x ) );
                    sibling = leftOf( parentOf( x ) );
                }
                if ( colorOf( rightOf( sibling ) ) == BLACK && colorOf( leftOf( sibling ) ) == BLACK )
                {
                    setColor( sibling, RED );
                    x = parentOf( x );
                }
                else
                {
                    if ( colorOf( leftOf( sibling ) ) == BLACK )
                    {
                        setColor( rightOf( sibling ), BLACK );
                        setColor( sibling, RED );
                        rotateLeft( sibling );
                        sibling = leftOf( parentOf( x ) );
                    }
                    setColor( sibling, colorOf( parentOf( x ) ) );
                    setColor( parentOf( x ), BLACK );
                    setColor( leftOf( sibling ), BLACK );
                    rotateRight( parentOf( x ) );
                    x = root;
                }


            }

        }
        setColor( x, BLACK );

    }

    public Node<V> getNode( V value )
    {
        Node<V> currentNode = root;
        while ( currentNode != null )
        {
            int cmp = value.compareTo( currentNode.value );
            if ( cmp == 0 )
            {
                return currentNode;
            }
            else if ( cmp < 0 )
            {
                currentNode = currentNode.left;
            }
            else
            {
                currentNode = currentNode.right;
            }
        }
        return null;
    }

    /**
     * return the largest entry, strictly smaller than the indicated value
     */
    public V getLargestBelow( V value )
    {
        Node<V> p = root;
        if ( p == null )
        {
            return null;
        }
        while ( true )
        {
            int cmp = value.compareTo( p.value );
            if ( cmp > 0 )
            {
                if ( p.right != null )
                {
                    p = p.right;
                }
                else
                {
                    return valueOf( p ); // p is the largest entry of them all with a lower value
                }
            }
            else
            {
                if ( p.left != null )
                {
                    p = p.left;
                }
                else
                {
                    Node<V> parent = p.parent;
                    Node<V> ch = p;
                    while ( parent != null && ch == parent.left )
                    {
                        ch = parent;
                        parent = parent.parent;
                    }
                    return valueOf( parent );
                }
            }
        }
    }

    private V valueOf( Node<V> node )
    {
        return node != null ? node.value : null;
    }

    public V getSuccessor( V value )
    {
        Node<V> node = getNode( value );
        return valueOf( getSuccessor( node ) );
    }

    public V last()
    {
        if ( root == null )
        {
            return null;
        }
        Node<V> last = root;
        while ( last.right != null )
        {
            last = last.right;
        }
        return last.value;
    }

    public Iterator<V> iterator()
    {
        Node<V> first = root;
        while ( first != null && first.left != null )
        {
            first = first.left;
        }
        final Node<V> initial = first;
        return new Iterator<V>()
        {
            private Node<V> next = initial;

            public boolean hasNext()
            {
                return next != null;
            }

            public V next()
            {
                Node<V> current = next;
                next = getSuccessor( current );
                return current.value;
            }

            public void remove()
            {
                throw new UnsupportedOperationException();
            }
        };
    }


    /**
     * return the largest entry, lesser than or equal to the indicated value
     */
    public Node<V> getSuccessor( Node<V> current )
    {
        if ( current == null )
        {
            return null;
        }
        else if ( current.right != null )
        {
            Node<V> p = current.right;
            while ( p.left != null )
            {
                p = p.left;
            }
            return p;
        }
        else
        {
            Node<V> p = current.parent;
            Node<V> child = current;
            while ( p != null && child == p.right )
            {
                child = p;
                p = p.parent;
            }
            return p;
        }
    }

    private void fixAfterInsertion( Node<V> node )
    {
        node.color = RED;
        while ( node != null && node != root && node.parent.color == RED )
        {
            if ( parentOf( node ) == leftOf( grandparentOf( node ) ) )
            {
                Node<V> y = rightOf( grandparentOf( node ) );
                if ( colorOf( y ) == RED )
                {
                    setColor( parentOf( node ), BLACK );
                    setColor( y, BLACK );
                    setColor( grandparentOf( node ), RED );
                    node = grandparentOf( node );
                }
                else
                {
                    if ( node == rightOf( parentOf( node ) ) )
                    {
                        node = parentOf( node );
                        rotateLeft( node );
                    }
                    setColor( parentOf( node ), BLACK );
                    setColor( grandparentOf( node ), RED );
                    if ( grandparentOf( node ) != null )
                    {
                        rotateRight( grandparentOf( node ) );
                    }
                }
            }
            else
            {
                Node<V> y = leftOf( grandparentOf( node ) );
                if ( colorOf( y ) == RED )
                {
                    setColor( parentOf( node ), BLACK );
                    setColor( y, BLACK );
                    setColor( grandparentOf( node ), RED );
                    node = grandparentOf( node );
                }
                else
                {
                    if ( node == leftOf( parentOf( node ) ) )
                    {
                        node = parentOf( node );
                        rotateRight( node );
                    }
                    setColor( parentOf( node ), BLACK );
                    setColor( grandparentOf( node ), RED );
                    if ( grandparentOf( node ) != null )
                    {
                        rotateLeft( grandparentOf( node ) );
                    }
                }
            }
        }
    }

    private void rotateLeft( Node<V> p )
    {
        Node<V> r = p.right;
        p.right = r.left;
        if ( r.left != null )
        {
            r.left.parent = p;
        }
        r.parent = p.parent;
        if ( p.parent == null )
        {
            root = r;
        }
        else if ( p.parent.left == p )
        {
            p.parent.left = r;
        }
        else
        {
            p.parent.right = r;
        }
        r.left = p;
        p.parent = r;
    }

    private void rotateRight( Node<V> p )
    {
        Node<V> l = p.left;
        p.left = l.right;
        if ( l.right != null )
        {
            l.right.parent = p;
        }
        l.parent = p.parent;
        if ( p.parent == null )
        {
            root = l;
        }
        else if ( p.parent.right == p )
        {
            p.parent.right = l;
        }
        else
        {
            p.parent.left = l;
        }
        l.right = p;
        p.parent = l;
    }

    private boolean colorOf( Node<V> node )
    {
        return node == null ? BLACK : node.color;
    }

    private void setColor( Node<V> node, boolean color )
    {
        if ( node != null )
        {
            node.color = color;
        }
    }

    private Node<V> grandparentOf( Node<V> node )
    {
        return parentOf( parentOf( node ) );
    }

    private Node<V> parentOf( Node<V> node )
    {
        return node != null ? node.parent : null;
    }

    private Node<V> leftOf( Node<V> node )
    {
        return node != null ? node.left : null;
    }

    private Node<V> rightOf( Node<V> node )
    {
        return node != null ? node.right : null;
    }

    static class Node<V extends Comparable<V>>
    {

        private Node( V value, Node<V> parent )
        {
            this.value = value;
            this.parent = parent;
        }

        V value;

        Node<V> parent;

        boolean color = BLACK;

        Node<V> left;

        Node<V> right;

        public String toString()
        {
            return "Node: " + value;
        }
    }
}
